==> new23d-lpep-0.0.3.tar.gz: new23d-lpep-0.0.3/bin/lpep.py <==
#!/usr/bin/python

# This work is licensed under the
# Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License .
# To view a copy of this license ,
# visit http://creativecommons.org/licenses/by-nc-sa/3.0/ .

import cProfile

def main():
	try:
		import sqlite3

		import ConfigParser
		import ldap
		import string

		c = ConfigParser.RawConfigParser()
		c.read('../manifests/lpep.ini')

		ldap_host = c.get('ldap', 'host')
		ldap_base = c.get('ldap', 'base')
		ldap_user = c.get('ldap', 'bind_dn')
		ldap_pass = c.get('ldap', 'bind_pw')

		groupsOfInterest = string.split(c.get('consider', 'groups'), ',')
		usersToIgnore = string.split(c.get('ignore', 'users'), ',')

		# advanced configuration
		db_file = 'lpep.db'

		# internals configuration
		db_user_version = 2315

		# start of code

		# processing some lists: trimming, lowercasing
		# processed groups of interest: pGoI
		pGoI = map(lambda _group: string.lower(string.strip(_group)), groupsOfInterest)
		# processed users to ignore: pUtI
		pUtI = map(lambda _user: string.lower(string.strip(_user)), usersToIgnore)

		# searching which groups specified exist in the directory
		groupsOfInterestLdapFilter = ''
		for _group in pGoI:
			groupsOfInterestLdapFilter += '(cn=%s)' % (_group)

		# opening an ldap handle
		ldaph = ldap.initialize('ldap://%s' % (ldap_host))
		ldaph.simple_bind_s(ldap_user, ldap_pass)
		ldaph.set_option(ldap.OPT_REFERRALS, 0)
		ldaph.set_option(ldap.OPT_TIMEOUT, 60)
		ldaph.set_option(ldap.OPT_NETWORK_TIMEOUT, 60)

		# limiting results by filtering for objectclass of group
		allGroupsList = ldaph.search_s(ldap_base, ldap.SCOPE_SUBTREE, '(&(objectclass=group)(|%s))' % (
									   groupsOfInterestLdapFilter), ['cn', 'primaryGroupToken', 'uSNCreated'])

		# obtain a list of all users. they will be filtered later down to the ones that are contained in one or more groups
		# of interest, but for now, get all accounts which have a login attribute like samaccountname present
		allUsersList = ldaph.search_s(ldap_base, ldap.SCOPE_SUBTREE, '(&(objectclass=user)(!(objectclass=computer))(sAMAccountName=*))',
									  ['sAMAccountName', 'primaryGroupID', 'memberOf', 'uSNCreated'])

		# removing referrals
		def remove_ldap_referrals(list):
			for _i in range(len(list) -1, -1, -1):
				if list[_i][0] == None:
					del list[_i]

		remove_ldap_referrals(allGroupsList)
		remove_ldap_referrals(allUsersList)

		# closing the ldap connection
		ldaph.unbind_ext_s()

		# sort by uSNCreated
		def sort_by_usncreated(key):
			return int(key[1]['uSNCreated'][0])

		# processed All Groups List: pAGL
		pAGL = sorted(allGroupsList, key=sort_by_usncreated)
		# process All Users List: pAUL
		# possibly unnecessary
		pAUL = sorted(allUsersList, key=sort_by_usncreated)

		# sort by user defined sequence of groups
		def sort_by_pgoi(key):
			return pGoI.index(string.lower(key[1]['cn'][0]))

		# processed groups of interest II, sorted by how the user specified them
		pGoI2 = sorted(allGroupsList, key=sort_by_pgoi)

		# freeing up some memory
		del allGroupsList
		del allUsersList

		# creating a user map with all groups as supplementary
		userMap = {}
		for _user in pAUL:

			# skipping users to ignore
			if string.lower(_user[1]['sAMAccountName'][0]) in pUtI:
				continue

			# create a user in the map
			userMap[_user[1]['sAMAccountName'][0]] = {'gid': None, 'groups': [], 'uSNCreated': None}

			# looking in the user's supplementary groups field
			if 'memberOf' in _user[1]:
				for _user_group_dn in _user[1]['memberOf']:
					for _group in pAGL:
						if _user_group_dn == _group[0]:
							userMap[_user[1]['sAMAccountName'][0]]['groups'].append(_group[1]['cn'][0])

			# looking at the user's primary group and merging it into the supplemental lot
			for _group in pAGL:
				if _group[1]['primaryGroupToken'][0] == _user[1]['primaryGroupID'][0]:
					userMap[_user[1]['sAMAccountName'][0]]['groups'].append(_group[1]['cn'][0])
					break

			# adding the uSNCreated information to run sort later
			userMap[_user[1]['sAMAccountName'][0]]['uSNCreated'] = int(_user[1]['uSNCreated'][0])

		# upgrading the primary group according to groupsOfInterest to primary/gid
		for _user in userMap:
			for _groupOfInterest in pGoI2:
				if _groupOfInterest[1]['cn'][0] in userMap[_user]['groups']:
					userMap[_user]['gid'] = _groupOfInterest[1]['cn'][0]
					del userMap[_user]['groups'][userMap[_user]['groups'].index(_groupOfInterest[1]['cn'][0])]
					break

		# find users without any groups at all...
		usersWithoutGroups = []
		for _user in userMap:
			if userMap[_user]['gid'] == None:
				usersWithoutGroups.append(_user)

		# ...do not consider them
		for _user in usersWithoutGroups:
			del userMap[_user]

		# sorting the usermap according to usncreated
		def sort_uid_by_usncreated(key):
			return int(userMap[key]['uSNCreated'])

		# function to translate group names to posix valid; spaces to underscores only for now
		trans_map = string.maketrans(' ', '_')
		def posixfy(name):
			return string.lower(string.translate(name, trans_map))

		# opening an sqlite handle
		sqlh = sqlite3.connect(db_file)
		sqlh.row_factory = sqlite3.Row
		sqlc = sqlh.cursor()
		sqlc.execute('PRAGMA foreign_keys = ON')

		# check if the schema is blank
		sqlco = sqlc.execute('PRAGMA schema_version')
		sqlr = sqlco.fetchone()
		schema_version = sqlr[0]

		if schema_version is 0:

			lpep_schema = '''
		PRAGMA foreign_keys = ON;

		CREATE TABLE g (
		  gid	INTEGER PRIMARY KEY AUTOINCREMENT,
		  name	VARCHAR NOT NULL UNIQUE
		);

		CREATE TABLE u (
		  uid	INTEGER PRIMARY KEY AUTOINCREMENT,
		  name	VARCHAR NOT NULL UNIQUE,
		  gid	INTEGER NOT NULL REFERENCES g(gid) ON DELETE CASCADE ON UPDATE CASCADE
		);

		CREATE TABLE sg (
		  gid	INTEGER NOT NULL REFERENCES g(gid) ON DELETE CASCADE ON UPDATE CASCADE,
		  uid	INTEGER NOT NULL REFERENCES u(uid) ON DELETE CASCADE ON UPDATE CASCADE
		);

		CREATE UNIQUE INDEX ix_sg_gid_uid ON sg(gid, uid);

		INSERT INTO sqlite_sequence (name, seq) VALUES ('g', 1000);
		INSERT INTO sqlite_sequence (name, seq) VALUES ('u', 1000);

		-- user_version should be set to Year + Day of the Year
		PRAGMA user_version = 2315;
			'''

			sqlc.executescript(lpep_schema)

		# check if the schema is as expected
		sqlco = sqlc.execute('PRAGMA user_version')
		sqlr = sqlco.fetchone()
		user_version = sqlr[0]

		if user_version != db_user_version:
			# database schema version not as expected
			print '2'
			exit(2)

		# populating sql db with groups
		for _group in pAGL:
			sqlc.execute('SELECT COUNT(*) FROM g WHERE name = :_name', {'_name': posixfy(_group[1]['cn'][0])})
			if sqlc.fetchone()[0] == 0:
				sqlc.execute('INSERT INTO g (name) VALUES (:_name)', {'_name': posixfy(_group[1]['cn'][0])})

		# populating sql db with users
		for _user in sorted(userMap, key=sort_uid_by_usncreated):
			sqlc.execute('SELECT COUNT(*) FROM u WHERE name = :_name', {'_name': posixfy(_user)})
			if sqlc.fetchone()[0] == 0:
				sqlc.execute('INSERT INTO u (name, gid) VALUES (:_name, (SELECT gid FROM g WHERE name = :_group_name))',
							 {'_name': posixfy(_user), '_group_name': posixfy(userMap[_user]['gid'])})

		# updating sql db with new [primary] gids for users
		for _user in sorted(userMap, key=sort_uid_by_usncreated):
			sqlc.execute('UPDATE u SET gid = (SELECT gid FROM g WHERE name = :_group_name) WHERE name = :_name AND \
						gid != (SELECT gid FROM g WHERE name = :_group_name)', {'_name': posixfy(_user),
						 '_group_name': posixfy(userMap[_user]['gid'])})

		# TODO code to only delete what has disappeared
		# truncating for now
		sqlc.execute('DELETE FROM sg')

		# and adding all the relations [again]
		for _user in sorted(userMap, key=sort_uid_by_usncreated):
			for _group in userMap[_user]['groups']:
				sqlc.execute('INSERT INTO sg (gid, uid) VALUES ((SELECT gid FROM g WHERE name = :_g_name), \
							(SELECT uid FROM u WHERE name = :_u_name))', {'_g_name': posixfy(_group),
							 '_u_name': posixfy(_user)})

		sqlh.commit()

		puppet_manifest = '''# This work is licensed under the
# Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License .
# To view a copy of this license ,
# visit http://creativecommons.org/licenses/by-nc-sa/3.0/ .

class lpep {
		'''

		sqlco = sqlc.execute('SELECT * FROM g')
		sqlr = sqlco.fetchall()

		for row in sqlr:
			puppet_manifest += '''
group { '%s' :
gid	=>	%s,
ensure	=>	present,
}
''' % (row['name'], row['gid'])

		sqlco = sqlc.execute('SELECT * FROM u')
		sqlr = sqlco.fetchall()
		for row in sqlr:

			puppet_group_members = ''
			sqlco2 = sqlc.execute('SELECT g.name from sg, g WHERE sg.uid = :_uid AND g.gid = sg.gid ORDER BY sg.gid', {'_uid': row['uid']})
			sqlr2 = sqlco2.fetchall()

			for row2 in sqlr2:
				puppet_group_members += "\n\t\t\t'" + row2['name'] + "',"

			puppet_manifest += '''
user { '%s' :
uid		=>	%s,
ensure		=>	present,
gid		=>	%s,
managehome	=>	true,
membership	=>	inclusive,
groups		=>	[%s
			],
}
''' % (row['name'], row['uid'], row['gid'], puppet_group_members)

		puppet_manifest += '''
}
'''

		# sql handle no longer needed
		sqlc.execute('VACUUM')
		sqlh.close()

		manifesth = open('../manifests/lpep.pp', 'w')
		manifesth.write(puppet_manifest)
		manifesth.close()

		print '0'
	except ():
		print sys.exc_info()
		print '4'
		exit(4)

cProfile.run('main()', 'profiling.log')

==> new23d-lpep-0.0.5.tar.gz: new23d-lpep-0.0.5/bin/lpep.py <==
#!/usr/bin/python

# This work is licensed under the
# Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License .
# To view a copy of this license ,
# visit http://creativecommons.org/licenses/by-nc-sa/3.0/ .

import cProfile

def main():
	try:
		import sqlite3

		import ConfigParser
		import ldap
		import string

		c = ConfigParser.RawConfigParser()
		c.read('../manifests/lpep.ini')

		ldap_host = c.get('ldap', 'host')
		ldap_base = c.get('ldap', 'base')
		ldap_user = c.get('ldap', 'bind_dn')
		ldap_pass = c.get('ldap', 'bind_pw')

		# ldap type: lt
		lt = c.get('ldap', 'type')

		groupsOfInterest = string.split(c.get('consider', 'groups'), ',')
		usersToIgnore = string.split(c.get('ignore', 'users'), ',')

		# advanced configuration
		db_file = 'lpep.db'

		# internals configuration
		db_user_version = 2315

		# ldap attributes for various ldap server types

		# username
		_la_uid = {}
		_la_uid['ad'] = 'sAMAccountName'
		_la_uid['domino'] = 'uid'

		# user objectclass
		_la_uoc = {}
		_la_uoc['ad'] = 'user'
		_la_uoc['domino'] = 'dominoPerson'

		# group objectclass
		_la_goc = {}
		_la_goc['ad'] = 'group'
		_la_goc['domino'] = 'dominoGroup'

		# memberOf
		_la_mo = {}
		_la_mo['ad'] = 'memberOf'
		_la_mo['domino'] = 'dominoaccessgroups'

		# uSNCreated
		_la_ct = {}
		_la_ct['ad'] = 'uSNCreated'
		_la_ct['domino'] = 'createtimestamp'

		# start of code

		# processing some lists: trimming, lowercasing
		# processed groups of interest: pGoI
		pGoI = map(lambda _group: string.lower(string.strip(_group)), groupsOfInterest)
		# processed users to ignore: pUtI
		pUtI = map(lambda _user: string.lower(string.strip(_user)), usersToIgnore)

		# searching which groups specified exist in the directory
		groupsOfInterestLdapFilter = ''
		for _group in pGoI:
			groupsOfInterestLdapFilter += '(cn=%s)' % (_group)

		# opening an ldap handle
		ldaph = ldap.initialize('ldap://%s' % (ldap_host))
		ldaph.simple_bind_s(ldap_user, ldap_pass)
		ldaph.set_option(ldap.OPT_REFERRALS, 0)
		ldaph.set_option(ldap.OPT_TIMEOUT, 60)
		ldaph.set_option(ldap.OPT_NETWORK_TIMEOUT, 60)

		# limiting results by filtering for objectclass of group
		allGroupsList = ldaph.search_s(ldap_base, ldap.SCOPE_SUBTREE, '(&(objectclass=%s)(|%s))'
									   % (_la_goc[lt], groupsOfInterestLdapFilter),
									   ['cn', 'primaryGroupToken', _la_ct[lt]])

		# obtain a list of all users. they will be filtered later down to the ones that are contained in one or more groups
		# of interest, but for now, get all accounts which have a login attribute like samaccountname present
		allUsersList = ldaph.search_s(ldap_base, ldap.SCOPE_SUBTREE, '(&(objectclass=%s)(!(objectclass=computer))(%s=*))'
									  % (_la_uoc[lt], _la_uid[lt]),
									  ['primaryGroupID', _la_uid[lt], _la_mo[lt], _la_ct[lt]])

		# removing referrals
		def remove_ldap_referrals(list):
			for _i in range(len(list) -1, -1, -1):
				if list[_i][0] == None:
					del list[_i]

		remove_ldap_referrals(allGroupsList)
		remove_ldap_referrals(allUsersList)

		# closing the ldap connection
		ldaph.unbind_ext_s()

		# sort by uSNCreated
		def sort_by_usncreated(key):
			return int(string.strip(key[1][_la_ct[lt]][0], string.letters))

		# processed All Groups List: pAGL
		pAGL = sorted(allGroupsList, key=sort_by_usncreated)
		# process All Users List: pAUL
		# possibly unnecessary
		pAUL = sorted(allUsersList, key=sort_by_usncreated)

		# sort by user defined sequence of groups
		def sort_by_pgoi(key):
			return pGoI.index(string.lower(key[1]['cn'][0]))

		# processed groups of interest II, sorted by how the user specified them
		pGoI2 = sorted(allGroupsList, key=sort_by_pgoi)

		# freeing up some memory
		del allGroupsList
		del allUsersList

		# creating a user map with all groups as supplementary
		userMap = {}
		for _user in pAUL:

			# skipping users to ignore
			if string.lower(_user[1][_la_uid[lt]][0]) in pUtI:
				continue

			# create a user in the map
			userMap[_user[1][_la_uid[lt]][0]] = {'gid': None, 'groups': [], _la_ct[lt]: None}

			# looking in the user's supplementary groups field
			if _la_ct[lt] in _user[1]:
				if _la_mo[lt] in _user[1]:
					for _user_group_dn in _user[1][_la_mo[lt]]:
						for _group in pAGL:
							if _user_group_dn == _group[0]:
								userMap[_user[1][_la_uid[lt]][0]]['groups'].append(_group[1]['cn'][0])

			if lt in ['ad',]:
				# looking at the user's primary group and merging it into the supplemental lot
				for _group in pAGL:
					if _group[1]['primaryGroupToken'][0] == _user[1]['primaryGroupID'][0]:
						userMap[_user[1][_la_uid[lt]][0]]['groups'].append(_group[1]['cn'][0])
						break

			# adding the uSNCreated information to run sort later
			userMap[_user[1][_la_uid[lt]][0]][_la_ct[lt]] = int(string.strip(_user[1][_la_ct[lt]][0], string.letters))

		# upgrading the primary group according to groupsOfInterest to primary/gid
		for _user in userMap:
			for _groupOfInterest in pGoI2:
				if _groupOfInterest[1]['cn'][0] in userMap[_user]['groups']:
					userMap[_user]['gid'] = _groupOfInterest[1]['cn'][0]
					del userMap[_user]['groups'][userMap[_user]['groups'].index(_groupOfInterest[1]['cn'][0])]
					break

		# find users without any groups at all...
		usersWithoutGroups = []
		for _user in userMap:
			if userMap[_user]['gid'] == None:
				usersWithoutGroups.append(_user)

		# ...do not consider them
		for _user in usersWithoutGroups:
			del userMap[_user]

		# sorting the usermap according to usncreated
		def sort_uid_by_usncreated(key):
			return int(userMap[key][_la_ct[lt]])

		# function to translate group names to posix valid; spaces to underscores only for now
		trans_map = string.maketrans(' ', '_')
		def posixfy(name):
			return string.lower(string.translate(name, trans_map))

		# opening an sqlite handle
		sqlh = sqlite3.connect(db_file)
		sqlh.row_factory = sqlite3.Row
		sqlc = sqlh.cursor()
		sqlc.execute('PRAGMA foreign_keys = ON')

		# check if the schema is blank
		sqlco = sqlc.execute('PRAGMA schema_version')
		sqlr = sqlco.fetchone()
		schema_version = sqlr[0]

		if schema_version is 0:

			lpep_schema = '''
		PRAGMA foreign_keys = ON;

		CREATE TABLE g (
		  gid	INTEGER PRIMARY KEY AUTOINCREMENT,
		  name	VARCHAR NOT NULL UNIQUE
		);

		CREATE TABLE u (
		  uid	INTEGER PRIMARY KEY AUTOINCREMENT,
		  name	VARCHAR NOT NULL UNIQUE,
		  gid	INTEGER NOT NULL REFERENCES g(gid) ON DELETE CASCADE ON UPDATE CASCADE
		);

		CREATE TABLE sg (
		  gid	INTEGER NOT NULL REFERENCES g(gid) ON DELETE CASCADE ON UPDATE CASCADE,
		  uid	INTEGER NOT NULL REFERENCES u(uid) ON DELETE CASCADE ON UPDATE CASCADE
		);

		CREATE UNIQUE INDEX ix_sg_gid_uid ON sg(gid, uid);

		INSERT INTO sqlite_sequence (name, seq) VALUES ('g', 1000);
		INSERT INTO sqlite_sequence (name, seq) VALUES ('u', 1000);

		-- user_version should be set to Year + Day of the Year
		PRAGMA user_version = 2315;
			'''

			sqlc.executescript(lpep_schema)

		# check if the schema is as expected
		sqlco = sqlc.execute('PRAGMA user_version')
		sqlr = sqlco.fetchone()
		user_version = sqlr[0]

		if user_version != db_user_version:
			# database schema version not as expected
			print '2'
			exit(2)

		# populating sql db with groups
		for _group in pAGL:
			sqlc.execute('SELECT COUNT(*) FROM g WHERE name = :_name', {'_name': posixfy(_group[1]['cn'][0])})
			if sqlc.fetchone()[0] == 0:
				sqlc.execute('INSERT INTO g (name) VALUES (:_name)', {'_name': posixfy(_group[1]['cn'][0])})

		# populating sql db with users
		for _user in sorted(userMap, key=sort_uid_by_usncreated):
			sqlc.execute('SELECT COUNT(*) FROM u WHERE name = :_name', {'_name': posixfy(_user)})
			if sqlc.fetchone()[0] == 0:
				sqlc.execute('INSERT INTO u (name, gid) VALUES (:_name, (SELECT gid FROM g WHERE name = :_group_name))',
							 {'_name': posixfy(_user), '_group_name': posixfy(userMap[_user]['gid'])})

		# updating sql db with new [primary] gids for users
		for _user in sorted(userMap, key=sort_uid_by_usncreated):
			sqlc.execute('UPDATE u SET gid = (SELECT gid FROM g WHERE name = :_group_name) WHERE name = :_name AND \
						gid != (SELECT gid FROM g WHERE name = :_group_name)', {'_name': posixfy(_user),
						 '_group_name': posixfy(userMap[_user]['gid'])})

		# TODO code to only delete what has disappeared
		# truncating for now
		sqlc.execute('DELETE FROM sg')

		# and adding all the relations [again]
		for _user in sorted(userMap, key=sort_uid_by_usncreated):
			for _group in userMap[_user]['groups']:
				sqlc.execute('INSERT INTO sg (gid, uid) VALUES ((SELECT gid FROM g WHERE name = :_g_name), \
							(SELECT uid FROM u WHERE name = :_u_name))', {'_g_name': posixfy(_group),
							 '_u_name': posixfy(_user)})

		sqlh.commit()

		puppet_manifest = '''# This work is licensed under the
# Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License .
# To view a copy of this license ,
# visit http://creativecommons.org/licenses/by-nc-sa/3.0/ .

class lpep {
		'''

		sqlco = sqlc.execute('SELECT * FROM g')
		sqlr = sqlco.fetchall()

		for row in sqlr:
			puppet_manifest += '''
group { '%s' :
gid	=>	%s,
ensure	=>	present,
}
''' % (row['name'], row['gid'])

		sqlco = sqlc.execute('SELECT * FROM u')
		sqlr = sqlco.fetchall()
		for row in sqlr:

			puppet_group_members = ''
			sqlco2 = sqlc.execute('SELECT g.name from sg, g WHERE sg.uid = :_uid AND g.gid = sg.gid ORDER BY sg.gid',
								  {'_uid': row['uid']})
			sqlr2 = sqlco2.fetchall()

			for row2 in sqlr2:
				puppet_group_members += "\n\t\t\t'" + row2['name'] + "',"

			puppet_manifest += '''
user { '%s' :
uid		=>	%s,
ensure		=>	present,
gid		=>	%s,
managehome	=>	true,
membership	=>	inclusive,
groups		=>	[%s
			],
}
''' % (row['name'], row['uid'], row['gid'], puppet_group_members)

		puppet_manifest += '''
}
'''

		# sql handle no longer needed
		sqlc.execute('VACUUM')
		sqlh.close()

		manifesth = open('../manifests/lpep.pp', 'w')
		manifesth.write(puppet_manifest)
		manifesth.close()

		print '0'
	except ():
		print sys.exc_info()
		print '4'
		exit(4)

cProfile.run('main()', 'profiling.log')
